@@ -32,41 +32,61 @@
 #define AA_MAY_LINK			(1 << 4)
 #define AA_MAY_LOCK			(1 << 5)
 #define AA_EXEC_MMAP			(1 << 6)
-#define AA_EXEC_UNSAFE			(1 << 7)
-#define AA_EXEC_MOD_0			(1 << 8)
-#define AA_EXEC_MOD_1			(1 << 9)
+#define AA_MAY_MOUNT			(1 << 7)
+#define AA_EXEC_UNSAFE			(1 << 8)
+#define AA_EXEC_INHERIT			(1 << 9)
+#define AA_EXEC_MOD_0			(1 << 10)
+#define AA_EXEC_MOD_1			(1 << 11)
+#define AA_EXEC_MOD_2			(1 << 12)
+#define AA_EXEC_MOD_3			(1 << 13)
 
 #define AA_BASE_PERMS			(AA_MAY_EXEC | AA_MAY_WRITE | \
 					 AA_MAY_READ | AA_MAY_APPEND | \
 					 AA_MAY_LINK | AA_MAY_LOCK | \
-					 AA_EXEC_MMAP | AA_EXEC_UNSAFE | \
-					 AA_EXEC_MOD_0 | AA_EXEC_MOD_1)
+					 AA_MAY_MOUNT | AA_EXEC_MMAP | \
+					 AA_EXEC_UNSAFE | AA_EXEC_INHERIT | \
+					 AA_EXEC_MOD_0 | AA_EXEC_MOD_1 | \
+					 AA_EXEC_MOD_2 | AA_EXEC_MOD_3)
+
 #define AA_USER_SHIFT			0
-#define AA_OTHER_SHIFT			10
+#define AA_OTHER_SHIFT			14
 
 #define AA_USER_PERMS			(AA_BASE_PERMS << AA_USER_SHIFT)
 #define AA_OTHER_PERMS			(AA_BASE_PERMS << AA_OTHER_SHIFT)
 
 #define AA_FILE_PERMS			(AA_USER_PERMS | AA_OTHER_PERMS )
 
-#define AA_CHANGE_PROFILE		(1 << 30)
-#define AA_ERROR_BIT			(1 << 31)
-#define AA_SHARED_PERMS			(AA_CHANGE_PROFILE | AA_ERROR_BIT)
+#define AA_USER_PTRACE			(1 << 28)
+#define AA_OTHER_PTRACE			(1 << 29)
+#define AA_PTRACE_PERMS			(AA_USER_PTRACE | AA_OTHER_PTRACE)
 
+#define AA_CHANGE_HAT			(1 << 30)
+#define AA_CHANGE_PROFILE		(1 << 31)
+#define AA_SHARED_PERMS			(AA_CHANGE_HAT | AA_CHANGE_PROFILE)
 
-#define AA_EXEC_MODIFIERS		(AA_EXEC_MOD_0 | AA_EXEC_MOD_1)
-#define AA_EXEC_TYPE			(AA_MAY_EXEC | AA_EXEC_UNSAFE | \
+#define AA_EXEC_MODIFIERS		(AA_EXEC_MOD_0 | AA_EXEC_MOD_1 | \
+					 AA_EXEC_MOD_2 | AA_EXEC_MOD_3)
+#define AA_EXEC_COUNT			16
+
+#define AA_USER_EXEC_MODIFIERS		(AA_EXEC_MODIFIERS << AA_USER_SHIFT)
+#define AA_OTHER_EXEC_MODIFIERS		(AA_EXEC_MODIFIERS << AA_OTHER_SHIFT)
+#define AA_ALL_EXEC_MODIFIERS		(AA_USER_EXEC_MODIFIERS | \
+					 AA_OTHER_EXEC_MODIFIERS)
+
+#define AA_EXEC_TYPE			(AA_EXEC_UNSAFE | AA_EXEC_INHERIT | \
 					 AA_EXEC_MODIFIERS)
 
-#define AA_EXEC_UNCONFINED		0
-#define AA_EXEC_INHERIT			(AA_EXEC_MOD_0)
+#define AA_EXEC_UNCONFINED		(AA_EXEC_MOD_0)
 #define AA_EXEC_PROFILE			(AA_EXEC_MOD_1)
-#define AA_EXEC_PROFILE_OR_INHERIT	(AA_EXEC_MOD_0 | AA_EXEC_MOD_1)
+#define AA_EXEC_LOCAL			(AA_EXEC_MOD_0 | AA_EXEC_MOD_1)
 
-#define AA_VALID_PERMS			(AA_FILE_PERMS | AA_CHANGE_PROFILE)
+#define AA_VALID_PERMS			(AA_FILE_PERMS | AA_PTRACE_PERMS | \
+					 AA_OTHER_PERMS)
 
-#define AA_EXEC_BITS			((AA_MAY_EXEC << AA_USER_SHIFT) | \
-					 (AA_MAY_EXEC << AA_OTHER_SHIFT))
+#define AA_USER_EXEC			(AA_MAY_EXEC << AA_USER_SHIFT)
+#define AA_OTHER_EXEC			(AA_MAY_EXEC << AA_OTHER_SHIFT)
+
+#define AA_EXEC_BITS			(AA_USER_EXEC | AA_OTHER_EXEC)
 
 #define ALL_AA_EXEC_UNSAFE		((AA_EXEC_UNSAFE << AA_USER_SHIFT) | \
 					 (AA_EXEC_UNSAFE << AA_OTHER_SHIFT))
@@ -74,6 +94,8 @@
 #define AA_USER_EXEC_TYPE		(AA_EXEC_TYPE << AA_USER_SHIFT)
 #define AA_OTHER_EXEC_TYPE		(AA_EXEC_TYPE << AA_OTHER_SHIFT)
 
+#define ALL_AA_EXEC_TYPE		(AA_USER_EXEC_TYPE | AA_OTHER_EXEC_TYPE)
+
 #define AA_LINK_BITS			((AA_MAY_LINK << AA_USER_SHIFT) | \
 					 (AA_MAY_LINK << AA_OTHER_SHIFT))
 
@@ -88,6 +110,13 @@
 				 (AA_LINK_SUBSET_TEST << AA_OTHER_SHIFT))
 #define LINK_TO_LINK_SUBSET(X)		(((X) << 1) & AA_LINK_SUBSET_TEST)
 
+
+/* Pack the audit, and quiet masks into a single 28 bit field in the
+ * format oq:oa:uq:ua
+ */
+#define PACK_AUDIT_CTL(audit, quiet)	(((audit) & 0x1fc07f) | \
+					 (((quiet) & 0x1fc07f) << 7))
+
 #define AA_HAT_SIZE	975	/* Maximum size of a subdomain
 					 * ident (hat) */
 #define AA_IP_TCP			0x0001
@@ -112,26 +141,23 @@ enum pattern_t {
 #define HAS_MAY_LINK(mode)		((mode) & AA_MAY_LINK)
 #define HAS_MAY_LOCK(mode)		((mode) & AA_MAY_LOCK)
 #define HAS_EXEC_MMAP(mode) 		((mode) & AA_EXEC_MMAP)
-#define HAS_EXEC_INHERIT(mode)		(((mode) & AA_EXEC_MODIFIERS) == \
-					 AA_EXEC_INHERIT)
-#define HAS_EXEC_PROFILE(mode)		(((mode) & AA_EXEC_MODIFIERS) == \
-					 AA_EXEC_PROFILE)
-#define HAS_EXEC_UNCONFINED(mode)	(((mode) & AA_EXEC_MODIFIERS) == \
-					 AA_EXEC_UNCONFINED)
-#define HAS_EXEC_PROFILE_OR_INHERIT(mode) (((mode) & AA_EXEC_MODIFIERS) == \
-					   AA_EXEC_PROFILE_OR_INHERIT)
+
 #define HAS_EXEC_UNSAFE(mode) 		((mode) & AA_EXEC_UNSAFE)
 #define HAS_CHANGE_PROFILE(mode)	((mode) & AA_CHANGE_PROFILE)
 
+#include <stdio.h>
 static inline int is_merged_x_consistent(int a, int b)
 {
 	if ((a & AA_USER_EXEC_TYPE) && (b & AA_USER_EXEC_TYPE) &&
 	    ((a & AA_USER_EXEC_TYPE) != (b & AA_USER_EXEC_TYPE)))
+{ fprintf(stderr, "failed user merge 0x%x 0x%x\n", a, b);
 		return 0;
+}
 	if ((a & AA_OTHER_EXEC_TYPE) && (b & AA_OTHER_EXEC_TYPE) &&
 	    ((a & AA_OTHER_EXEC_TYPE) != (b & AA_OTHER_EXEC_TYPE)))
+{ fprintf(stderr, "failed other merge 0x%x 0x%x\n", a, b);
 		return 0;
-
+}
 	return 1;
 }
 
