@@ -1,4 +1,4 @@
-/*   $Id: parser.h 1036 2007-11-16 09:37:31Z jrjohansen $ */
+/*   $Id: parser.h 1284 2008-06-09 12:48:13Z jrjohansen $ */
 
 /*
  *   Copyright (c) 1999, 2000, 2001, 2002, 2004, 2005, 2006, 2007
@@ -18,6 +18,7 @@
  */
 
 #include <netinet/in.h>
+#include <sys/resource.h>
 #include "pcre/internal.h"
 #include "immunix.h"
 #include "libapparmor_re/apparmor_re.h"
@@ -25,11 +26,17 @@
 typedef enum pattern_t pattern_t;
 
 struct flagval {
-  	int debug;
+  	int hat;
   	int complain;
   	int audit;
 };
 
+struct named_transition {
+	int present;
+	char *namespace;
+	char *name;
+};
+
 struct cod_pattern {
 	char *regex;		// posix regex
 	pcre *compiled;		// compiled regex, size is compiled->size
@@ -39,11 +46,15 @@ struct cod_entry {
 	char *namespace;
 	char *name;
 	char *link_name;
+	char *nt_name;
 	struct codomain *codomain; 	/* Special codomain defined
 					 * just for this executable */
 	int mode;			/* mode is 'or' of AA_* bits */
+	int audit;			/* audit flags for mode */
 	int deny;			/* TRUE or FALSE */
 
+	int subset;
+
 	pattern_t pattern_type;
 	struct cod_pattern pat;
 
@@ -59,20 +70,39 @@ struct aa_network_entry {
 	struct aa_network_entry *next;
 };
 
+struct aa_rlimits {
+	unsigned int specified;			/* limits that are set */
+	rlim_t limits[RLIMIT_NLIMITS];
+};
+
 struct codomain {
 	char *namespace;
 	char *name;				/* codomain name */
 	char *sub_name;				/* subdomain name or NULL */
 	int default_deny;			/* TRUE or FALSE */
+	int local;
+	int local_mode;				/* true if local, not hat */
+	int local_audit;
 
 	struct codomain *parent;
 
 	struct flagval flags;
 
 	unsigned int capabilities;
+	unsigned int audit_caps;
+	unsigned int deny_caps;
+	unsigned int quiet_caps;
+	unsigned int set_caps;
+
 	unsigned int *network_allowed;		/* array of type masks
 						 * indexed by AF_FAMILY */
+	unsigned int *audit_network;
+	unsigned int *deny_network;
+	unsigned int *quiet_network;
 
+	struct aa_rlimits rlimits;
+
+	char *exec_table[AA_EXEC_COUNT];
 	struct cod_entry *entries;
 	void *hat_table;
 	//struct codomain *next;
@@ -81,7 +111,7 @@ struct codomain {
 	int dfarule_count;
 	void *dfa;
 	size_t dfa_size;
-} ;
+};
 
 struct cod_global_entry {
 	struct cod_entry *entry;
@@ -124,6 +154,8 @@ struct var_string {
 #define COD_UNSAFE_UNCONFINED_CHAR	'u'
 #define COD_PROFILE_CHAR	'P'
 #define COD_UNSAFE_PROFILE_CHAR	'p'
+#define COD_LOCAL_CHAR		'C'
+#define COD_UNSAFE_LOCAL_CHAR	'c'
 
 #define OPTION_ADD      1
 #define OPTION_REMOVE   2
@@ -193,6 +225,7 @@ extern char *processquoted(char *string, int len);
 extern char *processunquoted(char *string, int len);
 extern int get_keyword_token(const char *keyword);
 extern int name_to_capability(const char *keyword);
+extern int get_rlimit(const char *name);
 extern char *process_var(const char *var);
 extern int parse_mode(const char *mode);
 extern struct cod_entry *new_entry(char *namespace, char *id, int mode,
@@ -220,6 +253,11 @@ extern char *get_next_set_value(void **context);
 extern void dump_symtab(void);
 extern void dump_expanded_symtab(void);
 
+/* parser_alias.c */
+extern int new_alias(const char *from, const char *to);
+extern void replace_aliases(void *list);
+extern void free_aliases(void);
+
 /* parser_merge.c */
 extern int codomain_merge_rules(struct codomain *cod);
 
@@ -228,11 +266,14 @@ typedef struct __sdserialize sd_serialize;
 extern int load_codomain(int option, struct codomain *cod);
 extern int sd_serialize_profile(sd_serialize *p, struct codomain *cod,
 				int flatten);
+extern int sd_load_buffer(int option, char *buffer, int size);
+
 
 /* parser_policy.c */
 extern void add_to_list(struct codomain *codomain);
 extern void add_hat_to_policy(struct codomain *policy, struct codomain *hat);
 extern void add_entry_to_policy(struct codomain *policy, struct cod_entry *entry);
+extern void post_process_nt_entries(struct codomain *cod);
 extern int post_process_policy(void);
 extern int process_hat_regex(struct codomain *cod);
 extern int process_hat_variables(struct codomain *cod);
