@@ -1,4 +1,4 @@
-/* $Id: parser_interface.c 1030 2007-11-16 09:34:01Z jrjohansen $ */
+/* $Id: parser_interface.c 1284 2008-06-09 12:48:13Z jrjohansen $ */
 
 /*
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
@@ -40,8 +40,9 @@
 #define _(s) gettext(s)
 
 #define u8  unsigned char
-#define u16 unsigned short
-#define u32 unsigned int
+#define u16 uint16_t
+#define u32 uint32_t
+#define u64 uint64_t
 
 #define BUFFERINC 65536
 //#define BUFFERINC 16
@@ -49,16 +50,18 @@
 #if __BYTE_ORDER == __BIG_ENDIAN
 #  define cpu_to_le16(x) ((u16)(bswap_16 ((u16) x)))
 #  define cpu_to_le32(x) ((u32)(bswap_32 ((u32) x)))
+#  define cpu_to_le64(x) ((u64)(bswap_64 ((u64) x)))
 #else
 #  define cpu_to_le16(x) ((u16)(x))
 #  define cpu_to_le32(x) ((u32)(x))
+#  define cpu_to_le64(x) ((u64)(x))
 #endif
 
 #define SD_CODE_SIZE (sizeof(u8))
 #define SD_STR_LEN (sizeof(u16))
 
 #define SUBDOMAIN_INTERFACE_VERSION 2
-#define SUBDOMAIN_INTERFACE_DFA_VERSION 3
+#define SUBDOMAIN_INTERFACE_DFA_VERSION 5
 
 int sd_serialize_codomain(int option, struct codomain *cod);
 
@@ -193,6 +196,8 @@ const char *sd_code_names[] = {
 	"SD_STRUCTEND",
 	"SD_LIST",
 	"SD_LISTEND",
+	"SD_ARRAY",
+	"SD_ARRAYEND",
 	"SD_OFFSET"
 };
 
@@ -206,7 +211,7 @@ struct __sdserialize {
 
 sd_serialize *alloc_sd_serial(void)
 {
-	sd_serialize *p = malloc(sizeof(sd_serialize));
+	sd_serialize *p = calloc(1, sizeof(sd_serialize));
 	if (!p)
 		return NULL;
 	p->buffer = malloc(BUFFERINC);
@@ -307,6 +312,17 @@ inline int sd_write32(sd_serialize *p, u32 b)
 	return 1;
 }
 
+inline int sd_write64(sd_serialize *p, u64 b)
+{
+	u64 tmp;
+	if (!sd_prepare_write(p, SD_U64, sizeof(b)))
+		return 0;
+	tmp = cpu_to_le64(b);
+	memcpy(p->pos, &tmp, sizeof(tmp));
+	sd_inc(p, sizeof(tmp));
+	return 1;
+}
+
 inline int sd_write_name(sd_serialize *p, char *name)
 {
 	long size = 0;
@@ -361,13 +377,11 @@ inline int sd_write_aligned_blob(sd_serialize *p, void *b, int buf_size,
 	return 1;
 }
 
-inline int sd_write_string(sd_serialize *p, char *b, char *name)
+static int sd_write_strn(sd_serialize *p, char *b, int size, char *name)
 {
 	u16 tmp;
-	int size;
 	if (!sd_write_name(p, name))
 		return 0;
-	size = strlen(b) + 1;
 	if (!sd_prepare_write(p, SD_STRING, SD_STR_LEN + size))
 		return 0;
 	tmp = cpu_to_le16(size);
@@ -378,6 +392,11 @@ inline int sd_write_string(sd_serialize *p, char *b, char *name)
 	return 1;
 }
 
+inline int sd_write_string(sd_serialize *p, char *b, char *name)
+{
+	return sd_write_strn(p, b, strlen(b) + 1, name);
+}
+
 inline int sd_write_struct(sd_serialize *p, char *name)
 {
 	if (!sd_write_name(p, name))
@@ -489,6 +508,66 @@ int sd_serialize_dfa(sd_serialize *p, void *dfa, size_t size)
 	return 1;
 }
 
+int sd_serialize_rlimits(sd_serialize *p, struct aa_rlimits *limits)
+{
+	int i;
+	if (!limits->specified)
+		return 1;
+	if (!sd_write_struct(p, "rlimits"))
+		return 0;
+	if (!sd_write32(p, limits->specified))
+		return 0;
+	if (!sd_write_array(p, NULL, RLIM_NLIMITS))
+		return 0;
+	for (i = 0; i < RLIM_NLIMITS; i++) {
+		if (!sd_write64(p, limits->limits[i]))
+			return 0;
+	}
+	if (!sd_write_arrayend(p))
+		return 0;
+	if (!sd_write_structend(p))
+		return 0;
+	return 1;
+}
+
+int sd_serialize_xtable(sd_serialize *p, char **table)
+{
+	int count, i;
+	if (!table[4])
+		return 1;
+	if (!sd_write_struct(p, "xtable"))
+		return 0;
+	count = 0;
+	for (i = 4; i < AA_EXEC_COUNT; i++) {
+		if (table[i])
+			count++;
+	}
+
+	if (!sd_write_array(p, NULL, count))
+		return 0;
+
+	for (i = 4; i < count + 4; i++) {
+		int len = strlen(table[i]) + 1;
+
+		/* if its a namespace make sure the second : is overwritten
+		 * with 0, so that the namespace and name are \0 seperated
+		 */
+		if (*table[i] == ':') {
+			char *tmp = table[i] + 1;
+			strsep(&tmp, ":");
+		}
+		if (!sd_write_strn(p, table[i], len, NULL))
+			return 0;
+
+	}
+	if (!sd_write_arrayend(p))
+		return 0;
+
+	if (!sd_write_structend(p))
+		return 0;
+	return 1;
+}
+
 int count_file_ents(struct cod_entry *list)
 {
 	struct cod_entry *entry;
@@ -529,6 +608,7 @@ int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
 			 int flattened)
 {
 	struct cod_entry *entry;
+	u32 allowed_caps;
 
 	if (!sd_write_struct(p, "profile"))
 		return 0;
@@ -552,7 +632,7 @@ int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
 	if (!sd_write_struct(p, "flags"))
 		return 0;
 	/* used to be flags.debug, but that's no longer supported */
-	if (!sd_write32(p, 0))
+	if (!sd_write32(p, profile->flags.hat))
 		return 0;
 	if (!sd_write32(p, profile->flags.complain))
 		return 0;
@@ -560,7 +640,17 @@ int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
 		return 0;
 	if (!sd_write_structend(p))
 		return 0;
-	if (!sd_write32(p, profile->capabilities))
+	allowed_caps = (profile->capabilities | profile->set_caps) & ~profile->deny_caps;
+	if (!sd_write32(p, allowed_caps))
+		return 0;
+	if (!sd_write32(p, allowed_caps & profile->audit_caps))
+		return 0;
+	if (!sd_write32(p, profile->deny_caps & profile->quiet_caps))
+		return 0;
+	if (!sd_write32(p, profile->set_caps & ~profile->deny_caps))
+		return 0;
+
+	if (!sd_serialize_rlimits(p, &profile->rlimits))
 		return 0;
 
 	if (profile->network_allowed) {
@@ -568,7 +658,13 @@ int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
 		if (!sd_write_array(p, "net_allowed_af", AF_MAX))
 			return 0;
 		for (i = 0; i < AF_MAX; i++) {
-			if (!sd_write16(p, profile->network_allowed[i]))
+		    u16 allowed = profile->network_allowed[i] &
+			~profile->deny_network[i];
+			if (!sd_write16(p, allowed))
+				return 0;
+			if (!sd_write16(p, allowed & profile->audit_network[i]))
+				return 0;
+			if (!sd_write16(p, profile->deny_network[i] & profile->quiet_network[i]))
 				return 0;
 		}
 		if (!sd_write_arrayend(p))
@@ -579,6 +675,9 @@ int sd_serialize_profile(sd_serialize *p, struct codomain *profile,
 	if (regex_type == AARE_DFA) {
 		if (!sd_serialize_dfa(p, profile->dfa, profile->dfa_size))
 			return 0;
+
+		if (!sd_serialize_xtable(p, profile->exec_table))
+			return 0;
 	} else {
 		/* pcre globbing entries */
 		if (count_pcre_ents(profile->entries)) {
@@ -787,3 +886,44 @@ int sd_serialize_codomain(int option, struct codomain *cod)
 exit:
 	return error;
 }
+
+int sd_load_buffer(int option, char *buffer, int size)
+{
+	int fd;
+	int error = 0, wsize;
+	char *filename = NULL;
+
+	switch (option) {
+	case OPTION_ADD:
+		asprintf(&filename, "%s/.load", subdomainbase);
+		fd = open(filename, O_WRONLY);
+		break;
+	case OPTION_REPLACE:
+		asprintf(&filename, "%s/.replace", subdomainbase);
+		fd = open(filename, O_WRONLY);
+		break;
+	default:
+		error = -EINVAL;
+		goto exit;
+		break;
+	}
+
+	if (fd < 0) {
+		PERROR(_("Unable to open %s - %s\n"), filename,
+		       strerror(errno));
+		error = -errno;
+		goto exit;
+	}
+
+	wsize = write(fd, buffer, size);
+	if (wsize < 0) {
+		error = -errno;
+	} else if (wsize < size) {
+		PERROR(_("%s: Unable to write entire profile entry\n"),
+		       progname);
+	}
+	close(fd);
+exit:
+	free(filename);
+	return error;
+}
