@@ -1,4 +1,4 @@
-/* $Id: parser_main.c 784 2007-07-18 15:22:59Z steve-beattie $ */
+/* $Id: parser_main.c 1288 2008-06-09 23:15:28Z jrjohansen $ */
 
 /*
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
@@ -59,6 +59,7 @@ const char *parser_copyright	= "Copyright (C) 1999, 2000, 2003, 2004, 2005, 2006
 char *progname;
 int option = OPTION_ADD;
 int force_complain = 0;
+int binary_input = 0;
 int names_only = 0;
 int dump_vars = 0;
 int dump_expanded_vars = 0;
@@ -67,11 +68,13 @@ char *subdomainbase = NULL;
 char *profilename;
 char *match_string = NULL;
 int regex_type = AARE_DFA;
+char *profile_namespace = NULL;
 
 extern int current_lineno;
 
 struct option long_options[] = {
 	{"add", 		0, 0, 'a'},
+	{"binary",		0, 0, 'B'},
 	{"base",		1, 0, 'b'},
 	{"debug",		0, 0, 'd'},
 	{"subdomainfs",		0, 0, 'f'},
@@ -85,10 +88,11 @@ struct option long_options[] = {
 	{"dump-expanded-variables",	0, 0, 'E'},
 	{"Include",		1, 0, 'I'},
 	{"remove",		0, 0, 'R'},
-	{"names",		0, 0, 'N'},	/* undocumented only emit profilenames */
+	{"names",		0, 0, 'N'},
 	{"stdout",		0, 0, 'S'},
 	{"match-string",	1, 0, 'm'},
 	{"quiet",		0, 0, 'q'},
+	{"namespace",		1, 0, 'n'},
 	{NULL, 0, 0, 0},
 };
 
@@ -107,19 +111,22 @@ static void display_usage(char *command)
 	       "Options:\n"
 	       "--------\n"
 	       "-a, --add		Add apparmor definitions [default]\n"
-	       "-d, --debug 		Debug apparmor definitions\n"
-	       "-h, --help		Display this text and exit\n"
 	       "-r, --replace		Replace apparmor definitions\n"
 	       "-R, --remove		Remove apparmor definitions\n"
-	       "-v, --version		Display version info and exit\n"
-	       "-p, --preprocess	Preprocess only\n"
 	       "-C, --Complain		Force the profile into complain mode\n"
-	       "-I n, --Include n	Add n to the search path\n"
+	       "-B, --binary		Input is precompiled profile\n"
+	       "-p, --preprocess	Dump profiles with includes expanded\n"
+	       "-N, --names		Dump names of profiles in input.\n"
+	       "-S, --stdout		Dump compiled profile to stdout\n"
 	       "-b n, --base n		Set base dir and cwd\n"
+	       "-I n, --Include n	Add n to the search path\n"
 	       "-f n, --subdomainfs n	Set location of apparmor filesystem\n"
-	       "-S, --stdout		Write output to stdout\n"
 	       "-m n, --match-string n  Use only match features n\n"
+	       "-n n, --namespace n	Set Namespace for the profile\n"
 	       "-q, --quiet		Don't emit warnings\n", command);
+	       "-v, --version		Display version info and exit\n"
+	       "-d, --debug 		Debug apparmor definitions\n"
+	       "-h, --help		Display this text and exit\n"
 }
 
 void pwarn(char *fmt, ...)
@@ -149,7 +156,7 @@ static int process_args(int argc, char *argv[])
 	int count = 0;
 	option = OPTION_ADD;
 
-	while ((c = getopt_long(argc, argv, "adf:hrRvpI:b:CNSm:q", long_options, &o)) != -1)
+	while ((c = getopt_long(argc, argv, "adf:hrRvpI:b:BCNSm:qn:", long_options, &o)) != -1)
 	{
 		switch (c) {
 		case 0:
@@ -190,6 +197,9 @@ static int process_args(int argc, char *argv[])
 		case 'b':
 			set_base_dir(optarg);
 			break;
+		case 'B':
+			binary_input =1;
+			break;
 		case 'C':
 			force_complain = 1;
 			break;
@@ -215,6 +225,9 @@ static int process_args(int argc, char *argv[])
 		case 'q':
 			conf_quiet = 1;
 			break;
+		case 'n':
+			profile_namespace = strdup(optarg);
+			break;
 		default:
 			display_usage(progname);
 			exit(0);
@@ -389,6 +402,52 @@ static int regex_support(void) {
 	return 0;
 }
 
+int process_binary(int option, char *profilename)
+{
+	char *buffer = NULL;
+	int retval = 0, size = 0, asize = 0, rsize;
+	int chunksize = 1 << 14;
+	int fd;
+
+	if (profilename) {
+		fd = open(profilename, O_RDONLY);
+		if (fd == -1) {
+			PERROR(_("Error: Could not read profile %s: %s.\n"),
+			       profilename, strerror(errno));
+			exit(errno);
+		}
+	} else {
+		fd = dup(0);
+	}
+
+	do {
+		if (asize - size == 0) {
+			buffer = realloc(buffer, chunksize);
+			asize = chunksize;
+			chunksize <<= 1;
+			if (!buffer) {
+				PERROR(_("Memory allocation error."));
+				exit(errno);
+			}
+		}
+
+		rsize = read(fd, buffer + size, asize - size);
+		if (rsize)
+			size += rsize;
+	} while (rsize > 0);
+
+	close(fd);
+
+	if (rsize == 0)
+		retval = sd_load_buffer(option, buffer, size);
+	else
+		retval = rsize;
+
+	free(buffer);
+
+	return retval;
+}
+
 int process_profile(int option, char *profilename)
 {
 	int retval = 0;
@@ -401,6 +460,11 @@ int process_profile(int option, char *profilename)
 	if (retval != 0)
 		goto out;
 
+	if (names_only) {
+		dump_policy_names();
+		goto out;
+	}
+
 	/* Get the match string to determine type of regex support needed */
 	get_match_string();
 
@@ -429,11 +493,6 @@ int process_profile(int option, char *profilename)
 		goto out;
 	}
 
-	if (names_only) {
-		dump_policy_names();
-		goto out;
-	}
-
 	if (!regex_support()) {
 		die_if_any_regex();
 	}
@@ -472,10 +531,12 @@ int main(int argc, char *argv[])
 		return retval;
 	}
 
-	parse_default_paths();
-	retval = process_profile(option, profilename);
-	if (retval != 0)
-		return retval;
+	if (binary_input) {
+		retval = process_binary(option, profilename);
+	} else {
+		parse_default_paths();
+		retval = process_profile(option, profilename);
+	}
 
 	return retval;
 }
