@@ -1,4 +1,4 @@
-/* $Id: parser_misc.c 1042 2007-11-29 18:06:53Z jrjohansen $ */
+/* $Id: parser_misc.c 1287 2008-06-09 22:17:41Z jrjohansen $ */
 
 /*
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
@@ -62,6 +62,41 @@ static struct keyword_table keyword_table[] = {
 	{"unsafe",		TOK_UNSAFE},
 	{"link",		TOK_LINK},
 	{"owner",		TOK_OWNER},
+	{"user",		TOK_OWNER},
+	{"other",		TOK_OTHER},
+	{"subset",		TOK_SUBSET},
+	{"audit",		TOK_AUDIT},
+	{"deny",		TOK_DENY},
+	{"profile",		TOK_PROFILE},
+	{"set",			TOK_SET},
+	{"rlimit",		TOK_RLIMIT},
+	{"alias",		TOK_ALIAS},
+	{"ptrace",		TOK_PTRACE},
+	/* terminate */
+	{NULL, 0}
+};
+
+static struct keyword_table rlimit_table[] = {
+	{"cpu",			RLIMIT_CPU},
+	{"fsize",		RLIMIT_FSIZE},
+	{"data",		RLIMIT_DATA},
+	{"stack",		RLIMIT_STACK},
+	{"core",		RLIMIT_CORE},
+	{"rss",			RLIMIT_RSS},
+	{"nofile",		RLIMIT_NOFILE},
+	{"ofile",		RLIMIT_OFILE},
+	{"as",			RLIMIT_AS},
+	{"nproc",		RLIMIT_NPROC},
+	{"memlock",		RLIMIT_MEMLOCK},
+	{"locks",		RLIMIT_LOCKS},
+	{"sigpending",		RLIMIT_SIGPENDING},
+	{"msgqueue",		RLIMIT_MSGQUEUE},
+#ifdef RLIMIT_NICE
+	{"nice",		RLIMIT_NICE},
+#endif
+#ifdef RLIMIT_RTPRIO
+	{"rtprio",		RLIMIT_RTPRIO},
+#endif
 	/* terminate */
 	{NULL, 0}
 };
@@ -102,6 +137,11 @@ int name_to_capability(const char *keyword)
 	return get_table_token("capability", capability_table, keyword);
 }
 
+int get_rlimit(const char *name)
+{
+	return get_table_token("rlimit", rlimit_table, name);
+}
+
 static struct keyword_table address_family[] = {
 /*	{"unix",	AF_UNIX},
 	{"local",	AF_LOCAL},	*/
@@ -407,7 +447,7 @@ static void warn_uppercase(void)
 static int parse_sub_mode(const char *str_mode, const char *mode_desc)
 {
 
-#define IS_DIFF_QUAL(mode, q) (((mode) & AA_MAY_EXEC) && (((mode) & (AA_EXEC_MODIFIERS | AA_EXEC_UNSAFE)) != (q)))
+#define IS_DIFF_QUAL(mode, q) (((mode) & AA_MAY_EXEC) && (((mode) & AA_EXEC_TYPE) != ((q) & AA_EXEC_TYPE)))
 
 	int mode = 0;
 	const char *p;
@@ -457,7 +497,7 @@ reeval:
 
 		case COD_INHERIT_CHAR:
 			PDEBUG("Parsing mode: found INHERIT\n");
-			if (IS_DIFF_QUAL(mode, AA_EXEC_INHERIT)) {
+			if (mode & AA_EXEC_MODIFIERS) {
 				yyerror(_("Exec qualifier 'i' invalid, conflicting qualifier already specified"));
 			} else {
 				if (next != tolower(next))
@@ -474,40 +514,48 @@ reeval:
 			      COD_UNSAFE_UNCONFINED_CHAR, COD_EXEC_CHAR);
 			/* fall through */
 		case COD_UNCONFINED_CHAR:
+			tmode |= AA_EXEC_UNCONFINED | AA_MAY_EXEC;
 			PDEBUG("Parsing mode: found UNCONFINED\n");
-			if (IS_DIFF_QUAL(mode, tmode | AA_EXEC_UNCONFINED)) {
+			if (IS_DIFF_QUAL(mode, tmode)) {
 				yyerror(_("Exec qualifier '%c' invalid, conflicting qualifier already specified"),
 					this);
 			} else {
 				if (next != tolower(next))
 					warn_uppercase();
-				mode |= tmode | AA_EXEC_UNCONFINED |
-				    AA_MAY_EXEC;
+				mode |=  tmode;
 				p++;	/* skip 'x' */
 			}
 			tmode = 0;
 			break;
 
 		case COD_UNSAFE_PROFILE_CHAR:
+		case COD_UNSAFE_LOCAL_CHAR:
 			tmode = AA_EXEC_UNSAFE;
 			/* fall through */
 		case COD_PROFILE_CHAR:
+		case COD_LOCAL_CHAR:
+			if (tolower(this) == COD_UNSAFE_PROFILE_CHAR)
+				tmode |= AA_EXEC_PROFILE | AA_MAY_EXEC;
+			else
+			{
+				tmode |= AA_EXEC_LOCAL | AA_MAY_EXEC;
+			}
 			PDEBUG("Parsing mode: found PROFILE\n");
 			if (tolower(next) == COD_INHERIT_CHAR) {
-				if (IS_DIFF_QUAL(mode, tmode | AA_EXEC_PROFILE_OR_INHERIT)) {
+				tmode |= AA_EXEC_INHERIT;
+				if (IS_DIFF_QUAL(mode, tmode)) {
 					yyerror(_("Exec qualifier '%c%c' invalid, conflicting qualifier already specified"), this, next);
 				} else {
-					mode |= tmode | AA_MAY_EXEC |
-					    AA_EXEC_PROFILE_OR_INHERIT;
+					mode |= tmode;
 					p += 2;		/* skip x */
 				}
-			} else if (IS_DIFF_QUAL(mode, tmode | AA_EXEC_PROFILE)) {
-				yyerror(_("Exec qualifier '%c' invalid, conflicting qualifier already specified"),
-					this);
+			} else if (IS_DIFF_QUAL(mode, tmode)) {
+				yyerror(_("Exec qualifier '%c' invalid, conflicting qualifier already specified"), this);
+
 			} else {
 				if (next != tolower(next))
 					warn_uppercase();
-				mode |= tmode | AA_EXEC_PROFILE | AA_MAY_EXEC;
+				mode |= tmode;
 				p++;	/* skip 'x' */
 			}
 			tmode = 0;
@@ -519,8 +567,11 @@ reeval:
 			break;
 
 		case COD_EXEC_CHAR:
-			PDEBUG("Parsing mode: found %s EXEC\n", mode_desc);
-			yyerror(_("Invalid mode, 'x' must be preceded by exec qualifier 'i', 'p', or 'u'"));
+			/* this is valid for deny rules, and named transitions
+			 * but invalid for regular x transitions
+			 * sort it out later.
+			 */
+			mode |= AA_MAY_EXEC;
 			break;
 
  		/* error cases */
@@ -571,7 +622,7 @@ struct cod_entry *new_entry(char *namespace, char *id, int mode, char *link_id)
 {
 	struct cod_entry *entry = NULL;
 
-	entry = (struct cod_entry *)malloc(sizeof(struct cod_entry));
+	entry = (struct cod_entry *)calloc(1, sizeof(struct cod_entry));
 	if (!entry)
 		return NULL;
 
@@ -579,6 +630,7 @@ struct cod_entry *new_entry(char *namespace, char *id, int mode, char *link_id)
 	entry->name = id;
 	entry->link_name = link_id;
 	entry->mode = mode;
+	entry->audit = 0;
 	entry->deny = FALSE;
 
 	entry->pattern_type = ePatternInvalid;
@@ -595,7 +647,7 @@ struct cod_entry *copy_cod_entry(struct cod_entry *orig)
 {
 	struct cod_entry *entry = NULL;
 
-	entry = (struct cod_entry *)malloc(sizeof(struct cod_entry));
+	entry = (struct cod_entry *)calloc(1, sizeof(struct cod_entry));
 	if (!entry)
 		return NULL;
 
@@ -646,20 +698,6 @@ static void debug_base_perm_mask(int mask)
 		printf("%c", COD_LINK_CHAR);
 	if (HAS_MAY_LOCK(mask))
 		printf("%c", COD_LOCK_CHAR);
-	if (HAS_EXEC_INHERIT(mask))
-		printf("%c", COD_INHERIT_CHAR);
-	if (HAS_EXEC_UNCONFINED(mask)) {
-		if (HAS_EXEC_UNSAFE(mask))
-			printf("%c", COD_UNSAFE_UNCONFINED_CHAR);
-		else
-			printf("%c", COD_UNCONFINED_CHAR);
-	}
-	if (HAS_EXEC_PROFILE(mask)) {
-		if (HAS_EXEC_UNSAFE(mask))
-			printf("%c", COD_UNSAFE_PROFILE_CHAR);
-		else
-			printf("%c", COD_PROFILE_CHAR);
-	}
 	if (HAS_EXEC_MMAP(mask))
 		printf("%c", COD_MMAP_CHAR);
 	if (HAS_MAY_EXEC(mask))
