@@ -1,4 +1,4 @@
-/* $Id: parser_policy.c 1030 2007-11-16 09:34:01Z jrjohansen $ */
+/* $Id: parser_policy.c 1279 2008-06-08 10:32:12Z jrjohansen $ */
 
 /*
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
@@ -42,7 +42,7 @@ void *policy_list = NULL;
 static int codomain_compare(const void *a, const void *b)
 {
 	struct codomain *A = (struct codomain *) a;
-	struct codomain *B = (struct codoamin *) b;
+	struct codomain *B = (struct codomain *) b;
 
 	int res = 0;
 	if (A->namespace) {
@@ -93,12 +93,101 @@ void add_hat_to_policy(struct codomain *cod, struct codomain *hat)
 	}
 }
 
+static int add_named_transition(struct codomain *cod, struct cod_entry *entry)
+{
+	char *name = NULL;
+	int i;
+
+	/* check to see if it is a local transition */
+	if (!entry->namespace) {
+		char *sub = strstr(entry->nt_name, "//");
+		/* does the subprofile name match the rule */
+
+		if (sub && strncmp(cod->name, sub, sub - entry->nt_name) &&
+		    strcmp(sub + 2, entry->name) == 0) {
+			free(entry->nt_name);
+			entry->nt_name = NULL;
+			return AA_EXEC_LOCAL >> 10;
+		} else if (((entry->mode & AA_USER_EXEC_MODIFIERS) ==
+			     SHIFT_MODE(AA_EXEC_LOCAL, AA_USER_SHIFT)) ||
+			    ((entry->mode & AA_OTHER_EXEC_MODIFIERS) ==
+			     SHIFT_MODE(AA_EXEC_LOCAL, AA_OTHER_SHIFT))) {
+			if (strcmp(entry->nt_name, entry->name) == 0) {
+				free(entry->nt_name);
+				entry->nt_name = NULL;
+				return AA_EXEC_LOCAL >> 10;
+			}
+			/* specified as cix so profile name is implicit */
+			name = malloc(strlen(cod->name) + strlen(entry->nt_name)
+				      + 3);
+			if (!name) {
+				PERROR("Memory allocation error\n");
+				exit(1);
+			}
+			sprintf(name, "%s//%s", cod->name, entry->nt_name);
+			free(entry->nt_name);
+			entry->nt_name = name;
+		}
+	}
+	if (entry->namespace) {
+		name = malloc(strlen(entry->namespace) + strlen(entry->nt_name) + 3);
+		if (!name) {
+			PERROR("Memory allocation error\n");
+			exit(1);
+		}
+		sprintf(name, ":%s:%s", entry->namespace, entry->nt_name);
+		free(entry->namespace);
+		free(entry->nt_name);
+		entry->namespace = NULL;
+		entry->nt_name = NULL;
+	} else {
+		name = entry->nt_name;
+	}
+
+	for (i = (AA_EXEC_LOCAL >> 10) + 1; i < AA_EXEC_COUNT; i++) {
+		if (!cod->exec_table[i]) {
+			cod->exec_table[i] = name;
+			return i;
+		} else if (strcmp(cod->exec_table[i], name) == 0) {
+			/* name already in table */
+			free(name);
+			return i;
+		}
+	}
+	free(name);
+	return 0;
+}
+
 void add_entry_to_policy(struct codomain *cod, struct cod_entry *entry)
 {
 	entry->next = cod->entries;
 	cod->entries = entry;
 }
 
+void post_process_nt_entries(struct codomain *cod)
+{
+	struct cod_entry *entry;
+
+	list_for_each(cod->entries, entry) {
+		if (entry->nt_name) {
+			int mode = 0;
+			int n = add_named_transition(cod, entry);
+			if (!n) {
+				PERROR("Profile %s has to many specified profile transitions.\n", cod->name);
+				exit(1);
+			}
+			if (entry->mode & AA_USER_EXEC)
+				mode |= SHIFT_MODE(n << 10, AA_USER_SHIFT);
+			if (entry->mode & AA_OTHER_EXEC)
+				mode |= SHIFT_MODE(n << 10, AA_OTHER_SHIFT);
+			entry->mode = ((entry->mode & ~AA_ALL_EXEC_MODIFIERS) |
+				       (mode & AA_ALL_EXEC_MODIFIERS));
+			entry->namespace = NULL;
+			entry->nt_name = NULL;
+		}
+	}
+}
+
 static void __merge_rules(const void *nodep, const VISIT value,
 			  const int __unused depth)
 {
@@ -230,6 +319,67 @@ int process_hat_variables(struct codomain *cod)
 	return 0;
 }
 
+
+static void __process_alias(const void *nodep, const VISIT value,
+			    const int __unused depth)
+{
+	struct codomain **t = (struct codomain **) nodep;
+
+	if (value == preorder || value == endorder)
+		return;
+
+	if ((*t)->entries)
+		replace_aliases((*t)->entries);
+
+	if ((*t)->hat_table)
+		twalk((*t)->hat_table, __process_alias);
+}
+
+int post_process_alias(void)
+{
+	twalk(policy_list, __process_alias);
+	return 0;
+}
+
+#define CHANGEHAT_PATH "/proc/[0-9]*/attr/current"
+
+/* add file rules to access /proc files to call change_hat()
+ */
+static void __add_hat_rules_parent(const void *nodep, const VISIT value,
+				   const int __unused depth)
+{
+	struct codomain **t = (struct codomain **) nodep;
+	struct cod_entry *entry;
+
+	if (value == preorder || value == endorder)
+		return;
+
+	/* don't add hat rules if a parent profile with no hats */
+	if (!(*t)->hat_table && !(*t)->parent)
+		return;
+
+	/* don't add hat rules for local_profiles */
+	if ((*t)->local)
+		return;
+
+	entry = new_entry(NULL, strdup(CHANGEHAT_PATH), AA_MAY_WRITE, NULL);
+	if (!entry) {
+		PERROR(_("ERROR adding hat access rule for profile %s\n"),
+		       (*t)->name);
+		exit(1);
+	}
+	add_entry_to_policy(*t, entry);
+
+	twalk((*t)->hat_table, __add_hat_rules_parent);
+}
+
+static int add_hat_rules(void)
+{
+	twalk(policy_list, __add_hat_rules_parent);
+
+	return 0;
+}
+
 /* Yuck, is their no other way to pass arguments to a twalk action */
 static int __load_option;
 
@@ -329,7 +479,11 @@ static void __dump_policy_hatnames(const void *nodep, const VISIT value,
 	if (value == preorder || value == endorder)
 		return;
 
-	printf("%s^%s\n", __dump_policy_name->name, (*t)->sub_name);
+	if (regex_type == AARE_DFA) {
+	    printf("%s//%s\n", __dump_policy_name->name, (*t)->name);
+	} else {
+	    printf("%s^%s\n", __dump_policy_name->name, (*t)->name);
+	}
 }
 
 void dump_policy_hatnames(struct codomain *cod)
@@ -386,6 +540,7 @@ struct codomain *merge_policy(struct codomain *a, struct codomain *b)
 {
 	struct codomain *ret = a;
 	struct cod_entry *last;
+
 	if (!a) {
 		ret = b;
 		goto out;
@@ -412,6 +567,21 @@ struct codomain *merge_policy(struct codomain *a, struct codomain *b)
 	a->flags.audit = a->flags.audit || b->flags.audit;
 
 	a->capabilities = a->capabilities | b->capabilities;
+	a->audit_caps = a->audit_caps | b->audit_caps;
+	a->deny_caps = a->deny_caps | b->deny_caps;
+	a->quiet_caps = a->quiet_caps | b->quiet_caps;
+	a->set_caps = a->set_caps | b->set_caps;
+
+	if (a->network_allowed) {
+		int i;
+		for (i = 0; i < AF_MAX; i++) {
+			a->network_allowed[i] |= b->network_allowed[i];
+			a->audit_network[i] |= b->audit_network[i];
+			a->deny_network[i] |= b->deny_network[i];
+			a->quiet_network[i] |= b->quiet_network[i];
+		}
+	}
+
 	merge_hats(a, b->hat_table);
 	tdestroy(b->hat_table, &empty_destroy);
 	b->hat_table = NULL;
@@ -425,6 +595,13 @@ int post_process_policy(void)
 {
 	int retval = 0;
 
+	retval = add_hat_rules();
+	if (retval != 0) {
+		PERROR(_("%s: Errors found during postprocessing.  Aborting.\n"),
+		       progname);
+		return retval;
+	}
+
 	retval = post_process_variables();
 	if (retval != 0) {
 		PERROR(_("%s: Errors found during regex postprocess.  Aborting.\n"),
@@ -432,6 +609,13 @@ int post_process_policy(void)
 		return retval;
 	}
 
+	retval = post_process_alias();
+	if (retval != 0) {
+		PERROR(_("%s: Errors found during postprocess.  Aborting.\n"),
+		       progname);
+		return retval;
+	}
+
 	retval = post_merge_rules();
 	if (retval != 0) {
 		PERROR(_("%s: Errors found in combining rules postprocessing. Aborting.\n"),
