@@ -1,4 +1,4 @@
-/* $Id: parser_regex.c 1042 2007-11-29 18:06:53Z jrjohansen $ */
+/* $Id: parser_regex.c 1277 2008-06-08 09:56:37Z jrjohansen $ */
 
 /*
  *   Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
@@ -496,46 +496,73 @@ static int process_dfa_entry(aare_ruleset_t *dfarules, struct cod_entry *entry)
 	/* ix implies m but the apparmor module does not add m bit to
 	 * dfa states like it does for pcre
 	 */
-	if (((entry->mode >> AA_OTHER_SHIFT) & AA_EXEC_MODIFIERS) ==
-	    AA_EXEC_INHERIT)
+	if ((entry->mode >> AA_OTHER_SHIFT) & AA_EXEC_INHERIT)
 		entry->mode |= AA_EXEC_MMAP << AA_OTHER_SHIFT;
-	if (((entry->mode >> AA_USER_SHIFT) & AA_EXEC_MODIFIERS) ==
-	    AA_EXEC_INHERIT)
+	if ((entry->mode >> AA_USER_SHIFT) & AA_EXEC_INHERIT)
 		entry->mode |= AA_EXEC_MMAP << AA_USER_SHIFT;
 
-	if (!aare_add_rule(dfarules, tbuf, entry->mode))
-		return FALSE;
+	/* relying on ptrace and change_profile not getting merged earlier */
+
+	/* the link bit on the first pair entry should not get masked
+	 * out by a deny rule, as both pieces of the link pair must
+	 * match.  audit info for the link is carried on the second
+	 * entry of the pair
+	 */
+	if (entry->deny && (entry->mode & AA_LINK_BITS)) {
+		if (!aare_add_rule(dfarules, tbuf, entry->deny,
+				   entry->mode & ~AA_LINK_BITS,
+				   entry->audit & ~AA_LINK_BITS))
+			return FALSE;
+	} else {
+		if (!aare_add_rule(dfarules, tbuf, entry->deny, entry->mode,
+				   entry->audit))
+			return FALSE;
+	}
+
 	if (entry->mode & (AA_LINK_BITS)) {
 		/* add the pair rule */
 		char lbuf[PATH_MAX + 8];
 		int perms = AA_LINK_BITS & entry->mode;
+		char *vec[2];
+		vec[0] = tbuf;
 		if (entry->link_name) {
-			sprintf(lbuf, "%s//%s", entry->name, entry->link_name);
+			ptype = convert_aaregex_to_pcre(entry->link_name, 0, lbuf, PATH_MAX + 8);
+			if (ptype == ePatternInvalid)
+				return FALSE;
+			if (entry->subset)
+				perms |= LINK_TO_LINK_SUBSET(perms);
+			vec[1] = lbuf;
 		} else {
 			perms |= LINK_TO_LINK_SUBSET(perms);
-			sprintf(lbuf, "%s///**", entry->name);
+			vec[1] = "/[^/].*";
 		}
-		ptype = convert_aaregex_to_pcre(lbuf, 0, tbuf, PATH_MAX + 8);
-		if (ptype == ePatternInvalid)
+		if (!aare_add_rule_vec(dfarules, entry->deny, perms, entry->audit & AA_LINK_BITS, 2, vec))
 			return FALSE;
-		if (!aare_add_rule(dfarules, tbuf, perms))
-			return FALSE;
-/*		if (!aare_add_vec_rule(dfarules, perms,
-				       tbuf, "/**", NULL))
-			return FALSE;
-*/
 	}
 	if (entry->mode & AA_CHANGE_PROFILE) {
-		char lbuf[2*PATH_MAX + 8];
-		if (entry->namespace)
-			sprintf(lbuf, "%s//%s", entry->namespace, entry->name);
-		else
-			sprintf(lbuf, "%s", entry->name);
-		ptype = convert_aaregex_to_pcre(lbuf, 0, tbuf, 2*PATH_MAX + 8);
-		if (ptype == ePatternInvalid)
-			return FALSE;
-		if (!aare_add_rule(dfarules, tbuf, AA_CHANGE_PROFILE))
-			return FALSE;
+		if (entry->namespace) {
+			char *vec[2];
+			vec[0] = entry->namespace;
+			vec[1] = entry->name;
+			if (!aare_add_rule_vec(dfarules, 0, AA_CHANGE_PROFILE, 0, 2, vec))
+			    return FALSE;
+		} else {
+			if (!aare_add_rule(dfarules, entry->name, 0, AA_CHANGE_PROFILE, 0))
+				return FALSE;
+		}
+	}
+	if (entry->mode & (AA_USER_PTRACE | AA_OTHER_PTRACE)) {
+		int mode = entry->mode & (AA_USER_PTRACE | AA_OTHER_PTRACE);
+		if (entry->namespace) {
+			char *vec[2];
+			vec[0] = entry->namespace;
+			vec[1] = entry->name;
+			if (!aare_add_rule_vec(dfarules, 0, mode, 0, 2, vec))
+			    return FALSE;
+		} else {
+			if (!aare_add_rule(dfarules, entry->name, 0, mode, 0))
+				return FALSE;
+		}
 	}
 	return TRUE;
 }
@@ -575,7 +602,9 @@ int process_regex(struct codomain *cod)
 
 	if (regex_type == AARE_DFA && cod->dfarule_count > 0) {
 		cod->dfa = aare_create_dfa(cod->dfarules, 0, &cod->dfa_size);
-		if (!cod->dfa)
+		aare_delete_ruleset(cod->dfarules);
+		cod->dfarules = NULL;
+ 		if (!cod->dfa)
 			goto out;
 /*
 		if (cod->dfa_size == 0) {
