@@ -1,5 +1,5 @@
 #! /bin/bash
-# $Id: net_raw.sh 61 2006-05-19 18:32:14Z steve-beattie $
+# $Id: net_raw.sh 1065 2007-12-23 01:06:09Z jrjohansen $
 
 #	Copyright (C) 2002-2005 Novell/SUSE
 #
@@ -28,6 +28,6 @@ genprofile
 runchecktest "RAW SOCKET (no cap)" fail
 
 # NET_RAW PASS (?)
-genprofile cap:net_raw
+genprofile cap:net_raw network:
 runchecktest "RAW SOCKET (cap net_raw)" pass
 
