@@ -1,5 +1,5 @@
 #! /bin/bash
-# $Id: open.sh 434 2007-03-13 12:32:04Z steve-beattie $
+# $Id: open.sh 1069 2007-12-23 01:10:29Z jrjohansen $
 
 #	Copyright (C) 2002-2007 Novell/SUSE
 #
@@ -55,4 +55,4 @@ runchecktest "OPEN R+dac_override" fail $file
 # to be required.
 rm -f ${file}
 genprofile $file:$badperm2
-runchecktest "OPEN W (create)" fail $file
+runchecktest "OPEN W (create)" known_fail $file
