@@ -1,5 +1,5 @@
 #! /bin/bash
-# $Id: readdir.sh 61 2006-05-19 18:32:14Z steve-beattie $
+# $Id: readdir.sh 1053 2007-12-23 01:00:56Z jrjohansen $
 
 #	Copyright (C) 2002-2005 Novell/SUSE
 #
@@ -30,12 +30,12 @@ mkdir $dir
 
 # CHDIR TEST
 
-genprofile $dir:$okperm
+genprofile $dir/:$okperm
 
 runchecktest "READDIR" pass $dir
 
 # CHDIR TEST (no perm)
 
-genprofile $dir:$badperm
+genprofile $dir/:$badperm
 
 runchecktest "READDIR (no perm)" fail $dir
